#include "ipSubsample.h"

namespace Torch {

ipSubsample::ipSubsample(int width_, int height_, char *coding_, int width_out_, int height_out_) : ipCore(coding_)
{
	width = width_;
        height = height_;

	width_out = width_out_;
        height_out = height_out_;

	real ratio_w = (real) width / width_out;
	real ratio_h = (real) height / height_out;
	if((ratio_w != (int) ratio_w) || (ratio_h != (int) ratio_h))
	{	
	   	warning("ipSubsample::process() incorrect ratio:\n");
	   	print("   width : %d / %d = %g\n", width, width_out, ratio_w);
	   	print("   height: %d / %d = %g\n", height, height_out, ratio_h);
	}

        seq_out = new(allocator) Sequence(1, width_out*height_out*n_planes);
}

void ipSubsample::process(Sequence *seq_in)
{
	real ratio_w = (real) width / width_out;
	real ratio_h = (real) height / height_out;
	
	switch(coding_id)
	{
	case 0:
		for(int h = 0; h < height_out; h++)
		{
			for(int w = 0; w < width_out; w++)
			{
		   		int h_ratio_h = (int) ((real) h * ratio_h);
			   	int w_ratio_w = (int) ((real) w * ratio_w);
				
				int h_ = h_ratio_h;
				int w_ = w_ratio_w;

				seq_out->frames[0][w + h * width_out] = seq_in->frames[0][w_ + h_ * width];
			}
		}
		break;
	case 1:
		for(int h = 0; h < height_out; h++)
		{
			for(int w = 0; w < width_out; w++)
			{
		   		int h_ratio_h = (int) ((real) h * ratio_h);
			   	int w_ratio_w = (int) ((real) w * ratio_w);
				
				int h_ = h_ratio_h;
				int w_ = w_ratio_w;

				int base_in = 3 * (w_ + h_ * width);
				int base_out = 3 * (w + h * width_out);
					
				seq_out->frames[0][base_out] = seq_in->frames[0][base_in];
				seq_out->frames[0][base_out+1] = seq_in->frames[0][base_in+1];
				seq_out->frames[0][base_out+2] = seq_in->frames[0][base_in+2];
			}
		}
		break;
	case 2:
	case 3:
		message("ipSubsample::process() not implemented for YUV and float format");
		break;
	}
}

int ipSubsample::getWidthOut(void)
{
	return width_out;
}

int ipSubsample::getHeightOut(void)
{
        return height_out;
}

void ipSubsample::setOutputSize(int width, int height)
{
	width_out = width;
	height_out = height;

	allocator->free(seq_out);
	seq_out = new(allocator) Sequence(1, width_out*height_out*n_planes);
}

ipSubsample::~ipSubsample()
{
}

}

