#include "pgmDiskXFile.h"

namespace Torch {

pgmDiskXFile::pgmDiskXFile(const char *file_name, const char *open_flags) : ImageDiskXFile(file_name, open_flags)
{
   	n_planes = 1;
	coding = "gray";
}

bool pgmDiskXFile::readHeader(int *width_, int *height_)
{
	char buffer[200];

   	// Reads Magic Number
	if(!gets(buffer, 200))
	{
		print("pgmDiskXFile::readHeader() [magic number] not enable to read pgm file %s\n", filename);
		return false;
	}
	if(strcmp(buffer, "P5\n") != 0)
	{
		print("pgmDiskXFile::readHeader() [magic number] incorrect magic number in file %s\n", filename);
		return false;
	}

	// Reads comments
	if(!gets(buffer, 200))
	{
		print("pgmDiskXFile::readHeader() [comments] not enable to read pgm file %s\n", filename);
		return false;
	}
	//while(buffer[0] == '#') 
	while(buffer[0] == '#' || buffer[0] == '\n') 
		if(!gets(buffer, 200)) 
		{
			print("pgmDiskXFile::readHeader() [comments] not enable to read pgm in file %s\n", filename);
			return false;
		}
	
	//
	// Reads header
	width = 0;
	height = 0;
	
   	sscanf(buffer, "%d %d\n", &width, &height);
	if(height == 0)
	{
		if(!gets(buffer, 200))
		{
			print("pgmDiskXFile::readHeader() [image size] not enable to read pgm file %s\n", filename);
			return false;
		}
   		sscanf(buffer, "%d\n", &height);
	}

   	if(width <= 0)
	{
		print("pgmDiskXFile::readHeader() incorrect width (%d) in pgm file %s\n", width, filename);
		return false;
	}
   	if(height <= 0)
	{
		print("pgmDiskXFile::readHeader() incorrect height (%d) in pgm file %s\n", height, filename);
		return false;
	}

	//
	// Reads resolution
	if(!gets(buffer, 200))
	{
		print("pgmDiskXFile::readHeader() [depth] not enable to read pgm file %s\n", filename);
		return false;
	}
	if(strcmp(buffer, "255\n") != 0)
	{
	   	print("pgmDiskXFile::readHeader() incorrect resolution (%s) in pgm file %s, only 255 is supported\n", buffer, filename);
		return false;
	}

	if(pixmap != NULL) delete [] pixmap;
	pixmap = new unsigned char [width * height];

	*width_ = width;
	*height_ = height;
	
	return true;
}

int pgmDiskXFile::readPixmap()
{
	return read(pixmap, sizeof(unsigned char), width*height);
}

bool pgmDiskXFile::writeHeader(int width_, int height_)
{
   	width = width_;
	height = height;

	printf("P5\n");
	printf("#\n");
	printf("# Image generated using Torch vision\n");
	printf("# (c) 2004-2005 Sebastien Marcel marcel@idiap.ch\n");
	printf("# IDIAP Research Institute\n");
	printf("#\n");
	printf("%d %d\n", width, height);
	printf("255\n");
   
	return true;
}

int pgmDiskXFile::writePixmap(unsigned char *pixmap_)
{
	return write(pixmap_, sizeof(unsigned char), width*height);
}

pgmDiskXFile::~pgmDiskXFile()
{
	if(pixmap != NULL) delete [] pixmap;
}

}
