const char *help = "\
progname: ppm2pgm.cc\n\
code2html: This program reads a ppm image and saves it as a pgm.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "ImageRgb.h"
#include "ImageGray.h"
#include "DiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
	char *image_filename;
	bool verbose;
  
	// Construct the command line
  	CmdLine cmd;
	cmd.setBOption("write log", false);
	
  	// Put the help line at the beginning
  	cmd.info(help);

  	cmd.addText("\nArguments:");
  	cmd.addSCmdArg("image filename", &image_filename, "image filename");
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");

	cmd.read(argc, argv);


  	DiskXFile *image_file = NULL;
	Image *colorimage = NULL;
	Image *grayimage = NULL;

	colorimage = new ImageRgb();
	colorimage->setBOption("verbose", verbose);
	
	image_file = new DiskXFile(image_filename, "r");
	colorimage->loadXFile(image_file);
	delete image_file;

	if(verbose)
	{
		print("Image info:\n");
		print("   width = %d\n", colorimage->width);
		print("   height = %d\n", colorimage->height);
		print("   format = %s (%d)\n", colorimage->coding, colorimage->n_planes);
	}

	grayimage = new ImageGray();
	grayimage->setBOption("verbose", verbose);

	grayimage->copyFrom(colorimage);
	//grayimage->copyFrom(colorimage->width, colorimage->height, colorimage->pixmap, colorimage->coding);
	//grayimage->copyFrom(colorimage->width, colorimage->height, colorimage->data, colorimage->coding);
	
	image_file = new DiskXFile("test.pgm", "w");
	grayimage->saveXFile(image_file);
	delete image_file;

	delete colorimage;
	delete grayimage;

	return(0);
}
