const char *help = "\
progname: zoompgm.cc\n\
code2html: This program reads a ppm image and zoom its content.\n\
version: Torch3 vision2.0, 2003-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "ImageRgb.h"
#include "ipZoom.h"
#include "DiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
	char *image_filename;
	bool verbose;
	int zoom;
  
	// Construct the command line
  	CmdLine cmd;
	cmd.setBOption("write log", false);
	
  	// Put the help line at the beginning
  	cmd.info(help);

  	cmd.addText("\nArguments:");
  	cmd.addSCmdArg("image filename", &image_filename, "image filename");
  	cmd.addICmdArg("zoom factor", &zoom, "zoom");
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");

	cmd.read(argc, argv);


  	DiskXFile *image_file = NULL;
	Image *image_in = NULL;
	Image *image_out = NULL;

	image_in = new ImageRgb();
	image_in->setBOption("verbose", verbose);
	
	image_file = new DiskXFile(image_filename, "r");
	image_in->loadXFile(image_file);
	delete image_file;

	if(verbose)
	{
		print("Image info:\n");
		print("   width = %d\n", image_in->width);
		print("   height = %d\n", image_in->height);
		print("   format = %s (%d)\n", image_in->coding, image_in->n_planes);
	}

	ipCore *ip = NULL;

	ip = new ipZoom(image_in->width, image_in->height, "rgb", zoom);
	ip->setBOption("verbose", verbose);
	ip->process(image_in);
	
	image_out = new ImageRgb();
	image_out->setBOption("verbose", verbose);

	image_out->copyFrom(ip->getWidthOut(), ip->getHeightOut(), ip->seq_out->frames[0], "rgb");
	
	image_file = new DiskXFile("zoom.ppm", "w");
	image_out->saveXFile(image_file);
	delete image_file;

	delete ip;
	delete image_out;
	delete image_in;

	return(0);
}
