#ifndef HAAR_PATTERN_DETECT_INC
#define HAAR_PATTERN_DETECT_INC

#include "PatternDetect.h"
#include "NonOrientedPatternFusion.h"

namespace Torch {
   
/** This class is designed to detect patterns in an image using a cascade of Haar-like classifiers
 
    @author Yann Rodiguez (rodrig@idiap.ch)
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class HaarPatternDetect : public PatternDetect
{
	//
	PatternMerger *merger;

	//
	NonOrientedPatternFusion *patternfusion;

	//
	real surfoverlap_fusion;

	//
	real threshold_activation;

public:
	///
	HaarPatternDetect(	const char *model_filename_,
      				real threshold_,
	      			int width_image_, int height_image_, 
				int patternWmin_, int patternHmin_, 
				int Wmin_, int Wmax_, real stepx_, real stepy_,
				real scale_factor_,
				real minMean_, real maxMean_, real minStdv_, real maxStdv_,
				real surfoverlap_fusion_,
				real threshold_activation_,
				real *pruning_ii_,
				real pruning_min_threshold_,
				real pruning_max_threshold_,
				int max_pattern=10000,
				bool verbose=false,
				bool realstump_ = false);

	///
	virtual void mergepatterns(void);

	///
	virtual ~HaarPatternDetect();
};

}

#endif
