#include "HaarRealStumpMachine.h"
#include "XFile.h"

namespace Torch {

HaarRealStumpMachine::HaarRealStumpMachine(int n_features_, ipHaar **masks_) : RealStumpMachine(n_features_, 100)
{
	//
	masks = masks_;
	
	//
	the_mask = NULL;
}

HaarRealStumpMachine::HaarRealStumpMachine(int width_pattern_, int height_pattern_, real *integralimage2_)
{
	the_mask = NULL;
        masks = NULL;
        
        width_pattern = width_pattern_;
        height_pattern = height_pattern_;
        integralimage2 = integralimage2_;

	// remark: feature_id, min, max, n_bins, lut initialized with the loadXFile method.
}
	
void HaarRealStumpMachine::setParams(int feature_id_, real min_, real max_, real *lut_)
{
	feature_id = feature_id_;
	min = min_;
	max = max_;
	lut = lut_;

	the_mask = masks[feature_id];
}
        
real HaarRealStumpMachine::getFeature(int feature_id_, Sequence *inputs_)
{
	if(masks != NULL) the_mask = masks[feature_id_];

	the_mask->process(inputs_);

	return the_mask->seq_out->frames[0][0];
}

void HaarRealStumpMachine::forward(Sequence *inputs)
{
	if(feature_id == -1)
		error("HaarRealStumpMachine::forward(): feature_id is not defined.");

	// computes the value of the feature #feature_id#
	real feature = getFeature(feature_id, inputs);

	if(feature <= min) 
	{
		outputs->frames[0][0] = lut[0];
		return;
	}
	if(feature >= max)
	{
		outputs->frames[0][0] = lut[n_bins-1];
		return;
	}
	
	int index = (int)floor(n_bins * (feature - min) / (max - min));

	outputs->frames[0][0] = lut[index];
}

void HaarRealStumpMachine::loadXFile(XFile *file)
{
   	file->taggedRead(&feature_id, sizeof(int), 1, "FEATURE");
	file->taggedRead(&min, sizeof(real), 1, "MIN");
	file->taggedRead(&max, sizeof(real), 1, "MAX");
	file->taggedRead(&n_bins, sizeof(int), 1, "N_BINS");
	
	lut = (real *)allocator->realloc(lut, sizeof(real)*n_bins);
	file->taggedRead(lut, sizeof(real), n_bins, "LUT");

	int mask_id_, x_, y_, w_, h_;
	file->taggedRead(&mask_id_, sizeof(int), 1, "MASK_ID");
        file->taggedRead(&x_, sizeof(int), 1, "X_MASK");
        file->taggedRead(&y_, sizeof(int), 1, "Y_MASK");
        file->taggedRead(&w_, sizeof(int), 1, "W_MASK");
        file->taggedRead(&h_, sizeof(int), 1, "H_MASK");

	switch(mask_id_)
	{
		case 1: /* Bi-Vertical */
			the_mask = new(allocator) biVertHaar(width_pattern, height_pattern, x_, y_, w_, h_); 
			break;
		case 2: /* Bi-Horizontal */
			the_mask = new(allocator) biHorizHaar(width_pattern, height_pattern, x_, y_, w_, h_);
			break;
		case 3: /* Diagonal */
			the_mask = new(allocator) diagHaar(width_pattern, height_pattern, x_, y_, w_, h_);
			break;
		case 4: /* Tri-Vertical */
			the_mask = new(allocator) triVertHaar(width_pattern, height_pattern, x_, y_, w_, h_);
			break;
		case 5: /* Tri-Horizontal */
			the_mask = new(allocator) triHorizHaar(width_pattern, height_pattern, x_, y_, w_, h_);
			break;
		case 6: /* Central */
			the_mask = new(allocator) centralHaar(width_pattern, height_pattern, x_, y_, w_, h_);
			break;
	}

	if(the_mask == NULL)
		error("HaarRealStumpMachine::loadXFile() impossible to allocate a Haar mask");

	the_mask->integralimage2 = integralimage2;
}

void HaarRealStumpMachine::saveXFile(XFile *file)
{
   	file->taggedWrite(&feature_id, sizeof(int), 1, "FEATURE");
	file->taggedWrite(&min, sizeof(real), 1, "MIN");
        file->taggedWrite(&max, sizeof(real), 1, "MAX");
	file->taggedWrite(&n_bins, sizeof(int), 1, "N_BINS");
        file->taggedWrite(lut, sizeof(real), n_bins, "LUT");

	the_mask->saveXFile(file);
}

HaarRealStumpMachine::~HaarRealStumpMachine()
{
}

}
