#ifndef LDA_MACHINE_INC
#define LDA_MACHINE_INC

#include "EigenMachine.h"

namespace Torch {

/** This class is designed to handle Linear Discriminant Analysis (LDA)

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0

    @see LDATrainer
*/
class LDAMachine : public EigenMachine
{  
public:
	/// projection of grand mean into LDA space
	real *d_m_mean_;

	/// normalize by #d_m_mean_#
	bool norm_d_mean;

	//-----

	///
	LDAMachine(int n_inputs_, bool norm_d_mean_ = true);	

	///
	virtual ~LDAMachine();

	/// computes the projection of the input onto the LDA matrix
	virtual void forward(Sequence *inputs);

	// load the parameters of the LDA
	virtual void loadXFile(XFile *file);

	// save the parameters of the LDA
	virtual void saveXFile(XFile *file);
};

}

#endif
