#include "Pattern.h"

namespace Torch {

Pattern::Pattern(int width_, int height_)
{
	scale = 0.0;
	view = 0;
	confidence = 0.0;
	activation = 0.0;
 	x = 0; 
	y = 0;	
	w = width_;
	h = height_;
}

void Pattern::reset()
{
	scale = 0.0;
	view = 0;
	confidence = 0.0;
	activation = 0.0;
	x = 0;
	y = 0;
	w = 0;
	h = 0;
}

void Pattern::sprint()
{
	print("x=%d y=%d w=%d h=%d scale=%2.3f view=%d confidence=%2.3f activation=%2.3f\n", x, y, w, h, scale, view, confidence, activation);
}

void Pattern::copyFrom(Pattern *pattern_)
{
	scale = pattern_->scale;
	view = pattern_->view;
	confidence = pattern_->confidence;
	activation = pattern_->activation;
	x = pattern_->x;
        y = pattern_->y;
        w = pattern_->w;
        h = pattern_->h;
}


// Comparing function to sort from smaller to bigger
int sortPatternSurfInc(const void *e1_, const void *e2_)
{
	Pattern **e1, **e2;
	Pattern *is1, *is2;
  
	e1 = (Pattern **) e1_;
	e2 = (Pattern **) e2_;
	is1 = (Pattern *) *e1;
	is2 = (Pattern *) *e2;

	if((is1->w * is1->h) > (is2->w * is2->h)) return 1;
	else if((is1->w * is1->h) < (is2->w * is2->h)) return -1;
	else return 0;
}

// Comparing function to sort from bigger to smaller
int sortPatternSurfDec(const void *e1_, const void *e2_)
{
	Pattern **e1, **e2;
	Pattern *is1, *is2;
  
	e1 = (Pattern **) e1_;
	e2 = (Pattern **) e2_;
	is1 = (Pattern *) *e1;
	is2 = (Pattern *) *e2;

	if((is1->w * is1->h) < (is2->w * is2->h)) return 1;
	else if((is1->w * is1->h) > (is2->w * is2->h)) return -1;
	else return 0;
}

// Comparing function to sort confidence from bigger to smaller
int sortPatternConfDec(const void *e1_, const void *e2_)
{
	Pattern **e1, **e2;
	Pattern *is1, *is2;
  
	e1 = (Pattern **) e1_;
	e2 = (Pattern **) e2_;
	is1 = (Pattern *) *e1;
	is2 = (Pattern *) *e2;

	if((is1->confidence) < (is2->confidence)) return 1;
	else if((is1->confidence) > (is2->confidence)) return -1;
	else return 0;
}

// Comparing function to sort activation from bigger to smaller
int sortPatternActivDec(const void *e1_, const void *e2_)
{
	Pattern **e1, **e2;
	Pattern *is1, *is2;
  
	e1 = (Pattern **) e1_;
	e2 = (Pattern **) e2_;
	is1 = (Pattern *) *e1;
	is2 = (Pattern *) *e2;

	if((is1->activation) < (is2->activation)) return 1;
	else if((is1->activation) > (is2->activation)) return -1;
	else return 0;
}

/// Return true if p1 is included in p2
bool rectIsInRect(Pattern *p1, Pattern *p2)
{
	int x_min, y_min;
	int x_max, y_max;
	int overlap_x_min, overlap_y_min;
	int overlap_x_max, overlap_y_max;

	if(p1->x <= p2->x)
	{
		x_min = p1->x;
		overlap_x_min = p2->x;
	}
	else
	{
		x_min = p2->x;
		overlap_x_min = p1->x;
	}

	
	if(p1->y <= p2->y)
	{
		y_min = p1->y;
		overlap_y_min = p2->y;
	}
	else
	{
		y_min = p2->y;
		overlap_y_min = p1->y;
	}

	if((p1->x + p1->w) >= (p2->x + p2->w))
	{
		x_max = p1->x + p1->w;
		overlap_x_max = p2->x + p2->w;
	}
	else
	{
		x_max = p2->x + p2->w;
		overlap_x_max = p1->x + p1->w;
	}


	if((p1->y + p1->h) >= (p2->y + p2->h))
	{
		y_max = p1->y + p1->h;
		overlap_y_max = p2->y + p2->h;
	}
	else
	{
		y_max = p2->y + p2->h;
		overlap_y_max = p1->y + p1->h;
	}

	if((overlap_x_max < overlap_x_min) || (overlap_y_max < overlap_y_min))
	{
		return false;	
	}
	else if(((x_max - x_min) > (p1->w + p2->w)) || ((y_max - y_min) > (p1->h + p2->h))) 
	{
	   	return false;
	}
	else 
	{
		if(p2->h * p2->w > p1->h * p1->w) return true;
		else return false;
	}
}

// Return the overlapping surface
real SurfOverlap(Pattern *p1, Pattern *p2)
{
	real surfOverlap;
	int x_min, y_min;
	int x_max, y_max;
	int overlap_x_min, overlap_y_min;
	int overlap_x_max, overlap_y_max;

	if(p1->x <= p2->x)
	{
		x_min = p1->x;
		overlap_x_min = p2->x;
	}
	else
	{
		x_min = p2->x;
		overlap_x_min = p1->x;
	}

	
	if(p1->y <= p2->y)
	{
		y_min = p1->y;
		overlap_y_min = p2->y;
	}
	else
	{
		y_min = p2->y;
		overlap_y_min = p1->y;
	}

	if((p1->x + p1->w) >= (p2->x + p2->w))
	{
		x_max = p1->x + p1->w;
		overlap_x_max = p2->x + p2->w;
	}
	else
	{
		x_max = p2->x + p2->w;
		overlap_x_max = p1->x + p1->w;
	}


	if((p1->y + p1->h) >= (p2->y + p2->h))
	{
		y_max = p1->y + p1->h;
		overlap_y_max = p2->y + p2->h;
	}
	else
	{
		y_max = p2->y + p2->h;
		overlap_y_max = p1->y + p1->h;
	}

	if((overlap_x_max < overlap_x_min) || (overlap_y_max < overlap_y_min))
	{
		surfOverlap = 0.0;
	}
	else if(((x_max - x_min) > (p1->w + p2->w)) || ((y_max - y_min) > (p1->h + p2->h))) 
	{
	   	surfOverlap = 0.0;
	}
	else 
	{
		if(p2->h * p2->w > p1->h * p1->w)
		{
			surfOverlap = ((real) (overlap_x_max - overlap_x_min) * (overlap_y_max - overlap_y_min)) / ((real) p2->h * p2->w);
		}
	   	else
	   	{
			surfOverlap = ((real) (overlap_x_max - overlap_x_min) * (overlap_y_max - overlap_y_min)) / ((real) p1->h * p1->w);
	    	}
	}
	
	return surfOverlap;
}

}

