#include "PatternCollector.h"

namespace Torch {

PatternCollector::PatternCollector(int width_, int height_, int max_patterns_)
{
	width = width_;
	height = height_;
	
	max_patterns = max_patterns_;
	if(max_patterns < 0)
		error("max_patterns (%d) must be positive!", max_patterns);
	
	n_patterns = 0;
	patterns = NULL;

	if(max_patterns > 0)
	{
		patterns = (Pattern **)allocator->alloc(sizeof(Pattern *) * max_patterns);
		for(int i = 0 ; i < max_patterns ; i++)
			patterns[i] = new(allocator) Pattern(width, height);
	}
}

bool PatternCollector::add(int x_, int y_, int w_, int h_, real confidence_, int view_)
{
   	if(max_patterns > 0)
	{
		if(n_patterns >= max_patterns)
		{
			warning("The maximum of patterns (%d) is reached.\n", max_patterns);
			return false;
		}

		Pattern *pattern_ = patterns[n_patterns];

		pattern_->x = x_;
		pattern_->y = y_;
		pattern_->w = w_;
		pattern_->h = h_;
		pattern_->scale = (real)w_ /width;
		pattern_->view = view_;
		pattern_->confidence = confidence_;
	}
	
	n_patterns++;

	return true;
}

PatternCollector::~PatternCollector()
{
}

}

