#include "RealStumpMachine.h"
#include "XFile.h"

namespace Torch {

RealStumpMachine::RealStumpMachine(int n_features_, int n_bins_)
{
	//
	n_bins = n_bins_;
	n_features = n_features_;
	
	//
	feature_id = -1;
	min = INF; max = -INF;
	lut = (real *)allocator->alloc(sizeof(real)*n_bins);

	//
        outputs = new(allocator) Sequence(1, 1);
}

void RealStumpMachine::setParams(int feature_id_, real min_, real max_, real *lut_)
{
	feature_id = feature_id_;
	min = min_;
	max = max_;
	lut = lut_;
}
        
real RealStumpMachine::getFeature(int feature_id_, Sequence *inputs_)
{
	return inputs_->frames[0][feature_id_]; 
}

void RealStumpMachine::forward(Sequence *inputs)
{
	if(feature_id == -1)
		error("RealStumpMachine::forward(): feature_id is not defined.");

	// computes the value of the feature #feature_id#
	real feature = getFeature(feature_id, inputs);

	if(feature <= min) 
	{
		outputs->frames[0][0] = lut[0];
		return;
	}
	if(feature >= max)
	{
		outputs->frames[0][0] = lut[n_bins-1];
		return;
	}
	
	int index = (int)floor(n_bins * (feature - min) / (max - min));

	outputs->frames[0][0] = lut[index];
}

void RealStumpMachine::loadXFile(XFile *file)
{
   	file->taggedRead(&feature_id, sizeof(int), 1, "FEATURE");
	file->taggedRead(&min, sizeof(real), 1, "MIN");
	file->taggedRead(&max, sizeof(real), 1, "MAX");
	file->taggedRead(&n_bins, sizeof(int), 1, "N_BINS");

	lut = (real *)allocator->realloc(lut, sizeof(real)*n_bins);
	file->taggedRead(lut, sizeof(real), n_bins, "LUT");
}

void RealStumpMachine::saveXFile(XFile *file)
{
   	file->taggedWrite(&feature_id, sizeof(int), 1, "FEATURE");
	file->taggedWrite(&min, sizeof(real), 1, "MIN");
        file->taggedWrite(&max, sizeof(real), 1, "MAX");
	file->taggedWrite(&n_bins, sizeof(int), 1, "N_BINS");
        file->taggedWrite(lut, sizeof(real), n_bins, "LUT");
}

RealStumpMachine::~RealStumpMachine()
{
}

}
