#ifndef STUMP_MACHINE_INC
#define STUMP_MACHINE_INC

#include "Machine.h"

namespace Torch {

/** This class is designed to train/support a (discrete) weak classifier

    It has three parameters:
    \begin{itemize}
    \item the index of the feature to test
    \item a threshold value to apply to the above feature
    \item the direction of the comparison (< or >) 
    	  \begin{itemize}
	  \item if (direction == -1) then if the value of the feature < threshold => positive pattern
	  \item if (direction == 1) the if the value of the feature > threshold => positive pattern
	  \item else -> negative pattern
    \end{itemize}
    
    @author Yann Rodriguez (yann.rodriguez@idiap.ch)
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class StumpMachine : public Machine
{   
protected:
	// explicit parameters of the machine 
	int feature_id;
	real threshold;
	int direction;

public:
        /// number of possible features
        int n_features;

	//-----
	
	///
	StumpMachine(int n_features_ = 0);	

	/// decode the parameters of the machine
	virtual void setParams(int feature_id_, real threshold_, int direction_);

	/// compute the feature #feature_id_# for the sequence #inputs_# 
        virtual real getFeature(int feature_id_, Sequence *inputs_);

	/// computes the output of the machine given an input sequence
	virtual void forward(Sequence *inputs);

	/// load the parameters of the machine
	virtual void loadXFile(XFile *file);

	/// save the parameters of the machine
	virtual void saveXFile(XFile *file);

	///	
	virtual ~StumpMachine();
};

}

#endif
