#include "SonyEviD100P.h"

namespace Torch {

SonyEviD100P::SonyEviD100P(char *device_, char port_) : ViscaSerial(device_, port_)
{
   	pan_speed = 1;
	tilt_speed = 1;

   	pan_min 	= 0xFA60;
        pan_max 	= 0x05A0;
	pan_center 	= 0x0000;
        tilt_min 	= 0xFE98;
        tilt_max 	= 0x0168;
	tilt_center 	= 0x0000;
}

SonyEviD100P::~SonyEviD100P()
{
}

int SonyEviD100P::getPowerStatus()
{
	sCommandPacket sndCommand, revCommand;

  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x09; 
  	sndCommand.packet[2]=0x04; 
	sndCommand.packet[3]=0x00;
  	sndCommand.packet[4]=0xff;
  	sndCommand.length=5;

  	// Send request directly
  	if(send(&sndCommand) < 0) return -1;

  	// Read the power status
  	if((waitingForReply(RETRIES, &revCommand) < 0) || (revCommand.length != 4)) 
	{
    		printf("\nCouldn't read the status of the power on the camera \n"); 
    		return -1;
  	}
  	else return(!(revCommand.packet[2]-2));
}

bool SonyEviD100P::setPowerOn()
{
	sCommandPacket sndCommand;

  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x04; 
	sndCommand.packet[3]=0x00;
	sndCommand.packet[4]=0x02;
  	sndCommand.packet[5]=0xff;
  	sndCommand.length=6;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::setPowerOff()
{
	sCommandPacket sndCommand;

  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x04; 
	sndCommand.packet[3]=0x00;
	sndCommand.packet[4]=0x03;
  	sndCommand.packet[5]=0xff;
  	sndCommand.length=6;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::setBackLightOn()
{
	sCommandPacket sndCommand;

  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x04; 
	sndCommand.packet[3]=0x33;
	sndCommand.packet[4]=0x02;
  	sndCommand.packet[5]=0xff;
  	sndCommand.length=6;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::setBackLightOff()
{
	sCommandPacket sndCommand;

  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x04; 
	sndCommand.packet[3]=0x33;
	sndCommand.packet[4]=0x03;
  	sndCommand.packet[5]=0xff;
  	sndCommand.length=6;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::goHome()
{
	sCommandPacket sndCommand;

  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
  	sndCommand.packet[3]=0x04; 
  	sndCommand.packet[4]=0xff; 
  	sndCommand.length=5;

  	// Send the command to the camera
  	return(sendCmd(&sndCommand));
}
	
bool SonyEviD100P::goRelative(int x, int y, int width, int height)
{
   	print("SonyEviD100P::goRelative() not implemnted\n");

	return false;
}

bool SonyEviD100P::goAbsolute(int x, int y, int width, int height)
{
	sCommandPacket sndCommand;

	char pan_pos[5];
	char tilt_pos[5];
	
	int pos;
	
//	print("SonyEviD100P::goAbsolute xy\n");

	pos = pan_min + (x * (pan_max-pan_min) / width);
	sprintf(pan_pos, "%04x", pos);
	
	printf(" + x=%d\tpos=%d\tpan_pos=%s\n", x, pos, pan_pos);
	
	pos = tilt_min + (y * (tilt_max-tilt_min) / height);
	sprintf(tilt_pos, "%04x", pos);

	print(" + y=%d\tpos=%d\ttilt_pos=%s\n", x, pos, tilt_pos);
	
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x02;
	sndCommand.packet[4]=pan_speed;
	sndCommand.packet[5]=tilt_speed;
	sndCommand.packet[6]=0xf&pan_pos[0];
	sndCommand.packet[7]=0xf&pan_pos[1];
	sndCommand.packet[8]=0xf&pan_pos[2];
	sndCommand.packet[9]=0xf&pan_pos[3];
	sndCommand.packet[10]=0xf&tilt_pos[0];
	sndCommand.packet[11]=0xf&tilt_pos[1];
	sndCommand.packet[12]=0xf&tilt_pos[2];
	sndCommand.packet[13]=0xf&tilt_pos[3];
  	sndCommand.packet[14]=0xff;
  	sndCommand.length=15;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}
	
bool SonyEviD100P::goRelative(int pan, int tilt)
{
	sCommandPacket sndCommand;

	int panSteps, tiltSteps;
  	float pan_factor,tilt_factor;

  	if(pan > 100) 
	{
		print("\nSetAbsPanTilt: You have tried to pan %d degrees\n",pan);
    		print("which is outside <-100 - +100>\n");
    		print("Therefore pan has been reset to 100\n\n");
    		pan = 100;
  	}
  
  	if(pan < -100) 
	{
    		print("\nSetAbsPanTilt: You have tried to pan %d degrees\n",pan);
    		print("which is outside <-100 - +100>\n");
    		print("Therefore pan has been reset to -100\n\n");
    		pan = -100;
  	}
   
  	if(tilt > 25) 
  	{
    		print("\nSetAbsPanTilt: You have tried to tilt %d degrees\n",tilt);
    		print("which is outside <-25 - +25>\n");
    		print("Therefore tilt has been reset to 25\n\n");
    		tilt = 25;
  	}
  
  	if(tilt < -25) 
  	{
    		print("\nSetAbsPanTilt: You have tried to tilt %d degrees\n",tilt);
    		print("which is outside <-25 - +25>\n");
    		print("Therefore tilt has been reset to -25\n\n");
    		tilt = -25;
  	} 

	float STEPSPERPANDEGLEFT  = 8.6;
	float STEPSPERPANDEGRIGHT  = 8.61;
	float STEPSPERTILTDEGUP = 11.44;
	float STEPSPERTILTDEGDOWN = 11.4;

  	if(pan > 0) pan_factor = STEPSPERPANDEGRIGHT;
  	else pan_factor = STEPSPERPANDEGLEFT;

  	if(tilt < 0) tilt_factor = STEPSPERTILTDEGDOWN;
  	else tilt_factor = STEPSPERTILTDEGUP;  
  
	panSteps =  0xffff & (int) (0x10000 + pan * pan_factor + 0.5);
  	tiltSteps = 0xffff & (int) (0x10000 + tilt * tilt_factor + 0.5);
  
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x03;
	sndCommand.packet[4]=getPanSpeedMax();
	sndCommand.packet[5]=getTiltSpeedMax();
	sndCommand.packet[6]=0xf & (panSteps >> 12);
	sndCommand.packet[7]=0xf & (panSteps >> 8);
	sndCommand.packet[8]=0xf & (panSteps >> 4);
	sndCommand.packet[9]=0xf &  panSteps;
	sndCommand.packet[10]=0xf & (tiltSteps >> 12);
	sndCommand.packet[11]=0xf & (tiltSteps >> 8);
	sndCommand.packet[12]=0xf & (tiltSteps >> 4);
	sndCommand.packet[13]=0xf & tiltSteps;
  	sndCommand.packet[14]=0xff;
  	sndCommand.length=15;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::goAbsolute(int pan, int tilt)
{
	sCommandPacket sndCommand;

	int panSteps, tiltSteps;
  	float pan_factor,tilt_factor;

  	if(pan > 100) 
	{
		print("\nSetAbsPanTilt: You have tried to pan %d degrees\n",pan);
    		print("which is outside <-100 - +100>\n");
    		print("Therefore pan has been reset to 100\n\n");
    		pan = 100;
  	}
  
  	if(pan < -100) 
	{
    		print("\nSetAbsPanTilt: You have tried to pan %d degrees\n",pan);
    		print("which is outside <-100 - +100>\n");
    		print("Therefore pan has been reset to -100\n\n");
    		pan = -100;
  	}
   
  	if(tilt > 25) 
  	{
    		print("\nSetAbsPanTilt: You have tried to tilt %d degrees\n",tilt);
    		print("which is outside <-25 - +25>\n");
    		print("Therefore tilt has been reset to 25\n\n");
    		tilt = 25;
  	}
  
  	if(tilt < -25) 
  	{
    		print("\nSetAbsPanTilt: You have tried to tilt %d degrees\n",tilt);
    		print("which is outside <-25 - +25>\n");
    		print("Therefore tilt has been reset to -25\n\n");
    		tilt = -25;
  	} 

	float STEPSPERPANDEGLEFT  = 8.6;
	float STEPSPERPANDEGRIGHT  = 8.61;
	float STEPSPERTILTDEGUP = 11.44;
	float STEPSPERTILTDEGDOWN = 11.4;

  	if(pan > 0) pan_factor = STEPSPERPANDEGRIGHT;
  	else pan_factor = STEPSPERPANDEGLEFT;

  	if(tilt < 0) tilt_factor = STEPSPERTILTDEGDOWN;
  	else tilt_factor = STEPSPERTILTDEGUP;  
  
	panSteps =  0xffff & (int) (0x10000 + pan * pan_factor + 0.5);
  	tiltSteps = 0xffff & (int) (0x10000 + tilt * tilt_factor + 0.5);
  
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x02;
	sndCommand.packet[4]=getPanSpeedMax();
	sndCommand.packet[5]=getTiltSpeedMax();
	sndCommand.packet[6]=0xf & (panSteps >> 12);
	sndCommand.packet[7]=0xf & (panSteps >> 8);
	sndCommand.packet[8]=0xf & (panSteps >> 4);
	sndCommand.packet[9]=0xf &  panSteps;
	sndCommand.packet[10]=0xf & (tiltSteps >> 12);
	sndCommand.packet[11]=0xf & (tiltSteps >> 8);
	sndCommand.packet[12]=0xf & (tiltSteps >> 4);
	sndCommand.packet[13]=0xf & tiltSteps;
  	sndCommand.packet[14]=0xff;
  	sndCommand.length=15;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}
	
bool SonyEviD100P::up()
{
	sCommandPacket sndCommand;
	
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x01;
	sndCommand.packet[4]=pan_speed;
	sndCommand.packet[5]=tilt_speed;
	sndCommand.packet[6]=0x03;
	sndCommand.packet[7]=0x01;
  	sndCommand.packet[8]=0xff;
  	sndCommand.length=9;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::down()
{
	sCommandPacket sndCommand;
	
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x01;
	sndCommand.packet[4]=pan_speed;
	sndCommand.packet[5]=tilt_speed;
	sndCommand.packet[6]=0x03;
	sndCommand.packet[7]=0x02;
  	sndCommand.packet[8]=0xff;
  	sndCommand.length=9;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::left()
{
	sCommandPacket sndCommand;
	
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x01;
	sndCommand.packet[4]=pan_speed;
	sndCommand.packet[5]=tilt_speed;
	sndCommand.packet[6]=0x01;
	sndCommand.packet[7]=0x03;
  	sndCommand.packet[8]=0xff;
  	sndCommand.length=9;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::right()
{
	sCommandPacket sndCommand;
	
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x01;
	sndCommand.packet[4]=pan_speed;
	sndCommand.packet[5]=tilt_speed;
	sndCommand.packet[6]=0x02;
	sndCommand.packet[7]=0x03;
  	sndCommand.packet[8]=0xff;
  	sndCommand.length=9;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::upLeft()
{
	sCommandPacket sndCommand;
	
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x01;
	sndCommand.packet[4]=pan_speed;
	sndCommand.packet[5]=tilt_speed;
	sndCommand.packet[6]=0x01;
	sndCommand.packet[7]=0x01;
  	sndCommand.packet[8]=0xff;
  	sndCommand.length=9;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::upRight()
{
	sCommandPacket sndCommand;
	
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x01;
	sndCommand.packet[4]=pan_speed;
	sndCommand.packet[5]=tilt_speed;
	sndCommand.packet[6]=0x02;
	sndCommand.packet[7]=0x01;
  	sndCommand.packet[8]=0xff;
  	sndCommand.length=9;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::downLeft()
{
	sCommandPacket sndCommand;
	
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x01;
	sndCommand.packet[4]=pan_speed;
	sndCommand.packet[5]=tilt_speed;
	sndCommand.packet[6]=0x01;
	sndCommand.packet[7]=0x02;
  	sndCommand.packet[8]=0xff;
  	sndCommand.length=9;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}

bool SonyEviD100P::downRight()
{
	sCommandPacket sndCommand;
	
  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x01;
	sndCommand.packet[4]=pan_speed;
	sndCommand.packet[5]=tilt_speed;
	sndCommand.packet[6]=0x02;
	sndCommand.packet[7]=0x02;
  	sndCommand.packet[8]=0xff;
  	sndCommand.length=9;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}


bool SonyEviD100P::stopPanTilt()
{
	sCommandPacket sndCommand;

  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x06; 
	sndCommand.packet[3]=0x01;
	sndCommand.packet[4]=pan_speed;
	sndCommand.packet[5]=tilt_speed;
	sndCommand.packet[6]=0x03;
	sndCommand.packet[7]=0x03;
  	sndCommand.packet[8]=0xff;
  	sndCommand.length=9;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}


bool SonyEviD100P::zoomTeleStd()
{
	sCommandPacket sndCommand;

  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x04; 
	sndCommand.packet[3]=0x07;
	sndCommand.packet[4]=0x02;
  	sndCommand.packet[5]=0xff;
  	sndCommand.length=6;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}
	
bool SonyEviD100P::zoomWideStd()
{
	sCommandPacket sndCommand;

  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x04; 
	sndCommand.packet[3]=0x07;
	sndCommand.packet[4]=0x03;
  	sndCommand.packet[5]=0xff;
  	sndCommand.length=6;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}
	
bool SonyEviD100P::stopZoom()
{
	sCommandPacket sndCommand;

  	// Camera and command id
  	sndCommand.packet[0]=0x81; 
	sndCommand.packet[1]=0x01; 
  	sndCommand.packet[2]=0x04; 
	sndCommand.packet[3]=0x07;
	sndCommand.packet[4]=0x00;
  	sndCommand.packet[5]=0xff;
  	sndCommand.length=6;

  	// Send request directly
  	if(send(&sndCommand) < 0) return false;
	else return true;
}
	
}

