#ifndef VIDEO_GRABBER_INC
#define VIDEO_GRABBER_INC

#include "VideoSource.h"

namespace Torch {

/** This class is designed to grab images from a video source (frame grabber, USB or Firewire).

    @author Sebastien Marcel (marcel@idiap.ch) 
    @version 2.0
    \Date
    @since 1.0
*/
class VideoGrabber : public VideoSource
{
public:
	/// constructor
  	VideoGrabber(bool verbose_ = false) : VideoSource(verbose_) { verbose = verbose_; };
  
	/// destructor
  	virtual ~VideoGrabber() {};

	/// return the brightness
  	virtual int getBrightness() = 0;

	/// change the brightness
  	virtual bool setBrightness(int) = 0;

	/// return the contrast
  	virtual int getContrast() = 0;

	/// change the contrast
  	virtual bool setContrast(int) = 0;
 
	/// return the hue
  	virtual int getHue() = 0;

	/// change the hue
  	virtual bool setHue(int) = 0;
 
	/// return the color
  	virtual int getColor() = 0;

	/// change the color
  	virtual bool setColor(int) = 0;
 
	/// return the gamma 
  	virtual int getGamma() = 0;

	/// change the gamma
  	virtual bool setGamma(int) = 0;
};

struct VideoGrabber_output_format_encode_
{
	char *image_coding;
	bool (*video_decode_output_encode)(int, int, unsigned char *, int, unsigned char *, int);
};

struct VideoGrabber_video_format_decode_
{
	char *image_coding;
	int bytesperline;
	int NbOutputFormatEncode;
	VideoGrabber_output_format_encode_ *output_format_encode;
};

}

#endif
